#!/usr/local/bin/webwork-perl

## $Id: profChangeDates.pl,v 1.8 2002/09/06 19:34:54 jj Exp $

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################


## This file is profChangeDates.pl
## It provides access to utilities for building, correcting and viewing problem sets
##


use lib '.'; use webworkInit; # WeBWorKInitLine
require 5.001;
use strict;


use Global;
use Auth;
use TimeLocal;
use CGI qw(:standard);


# Timing code
use Benchmark;
my $beginTime = new Benchmark;
# end Timing code

# variables global to this file
my ($setNumber,$setHeaderFileName,$probHeaderFileName,$dueDate,$openDate,$answerDate,$problemListref,$problemValueListref,$problemAttemptLimitListref);
my (@problemList,@problemValueList,@problemAttemptLimitList);

&CGI::ReadParse;
my %inputs=%main::in;

my ($classID, $fileName);

# establish environment for this script

    my $Course = $inputs{'course'};
&Global::getCourseEnvironment($Course);


my $cgiURL   		        = getWebworkCgiURL($Course);
my $courseScriptsDirectory 	= getCourseScriptsDirectory($Course);
my $databaseDirectory 		= getCourseDatabaseDirectory($Course);
my $htmlURL       			= getCourseHtmlURL($Course);
my $scriptDirectory   		= getWebworkScriptDirectory($Course);
my $templateDirectory       = getCourseTemplateDirectory;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";

my $updateMethod =$inputs{'updateMethod'};

# log access
	&Global::log_info('', query_string);

&webDatePage;

# begin Timing code
my $endTime = new Benchmark;
&Global::logTimingInfo($beginTime,$endTime,"profChangeDates.pl",$inputs{'course'},$inputs{'course'});
# end Timing code
exit;

#####################################END######################################

sub webDatePage {
#   verify the identity of the user.
	my $keyFile = &Global::getCourseKeyFile($inputs{'course'});
	&verify_key($inputs{'user'}, $inputs{'key'}, "$keyFile", $inputs{'course'});
	my $permissionsFile = &Global::getCoursePermissionsFile($inputs{'course'});
	my $permissions = &get_permissions($inputs{'user'}, $permissionsFile);
	if ($permissions != $Global::instructor_permissions ) {
		print "permissions = $permissions instructor_permissions= $Global::instructor_permissions\n";
		print &html_NO_PERMISSION;
		exit(0);
		}
#    if (!defined($inputs{'pCD'})) {
#		print &htmlTOP("Change Open/Due/Answer Date");
#        # print navigation buttons
#        print qq!
#        <A HREF="${cgiURL}profLogin.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
#        <IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
#        !;
#
#		&printProbSetForm;
#		print &htmlBOTTOM("profChangeDates.pl", \%inputs);
#		exit;
#   }

    if ($inputs{'pCD'}==1) {
		print &htmlTOP("Change Open/Due/Answer Date: Page 2");
        # print navigation buttons
        print qq!
        <A HREF="${cgiURL}profLogin.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
        <IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
        !;
		$setNumber=$inputs{'setNo'};
		&printForms($setNumber);
		print &htmlBOTTOM("profChangeDates.pl", \%inputs);
    }

    else {

		print &htmlTOP("Change Open/Due/Answer Date: Page 3");
        # print navigation buttons
        print qq!
        <A HREF="${cgiURL}profLogin.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
        <IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
        !;
		if ($inputs{'dateMode'} eq 'READWRITE') {
			&changeDates();
			&sendBack;
		}
		else {
			print '<H2>Read only mode, no changes made </H2>';
			print 'Go back and select Read/Write mode if you want to save changes.';
			&sendBack;
    	}
        print &htmlBOTTOM("profChangeDates.pl", \%inputs);
	}
}





#sub printProbSetForm {
#    print "<form action=\"${Global::cgiURL}profChangeDates.pl\">\n";
#
#    my %setNumberHash=&getAllProbSetNumbersHash;
#    &printProbSets("setNo",\%setNumberHash);
#
#    print "<input type=\"hidden\" name=\"pCD\" value=1>\n";
#    print &sessionKeyInputs(\%inputs);
#    print "<br><input type=\"submit\" value=\"Continue\"></form>";
#}

sub printForms {
    my ($setNumber) = @_;
    print qq!<FORM method="post" action="profChangeDates.pl">\n!;
    &printChangeOpenForm(&findOpenDateForSet($setNumber));
    &printChangeDueForm( &findDueDateForSet($setNumber));
    &printChangeAnswerForm(&findAnswerDateForSet($setNumber));

	&printChangeSetHeaderFileForm(&findSetHeaderFileForSet($setNumber));
	&printChangeProbHeaderFileForm(&findProbHeaderFileForSet($setNumber));


    print  qq!
    <TABLE><TR>\n<TD>
    Read Only<INPUT TYPE="RADIO" NAME="dateMode" VALUE="READONLY" CHECKED>\n
    </TD><TD>
	Read/Write<INPUT TYPE="RADIO" NAME="dateMode" VALUE="READWRITE">\n
	</TD><TD>
	<input type=submit value="Save Above Changes">\n
	</TD></TR></TABLE>\n
    <input type=hidden name="pCD" value=2>\n
    <input type=hidden name="setNo" value=$setNumber>\n
	!;
    print &sessionKeyInputs(\%inputs);
    print "</FORM>\n";

    &printChangeProblemFileNamesForm();


}


sub changeDates {
    my $newOpenDate=$inputs{'openDate'};
    my $newDueDate=$inputs{'dueDate'};
    my $newAnswerDate=$inputs{'ansDate'};
    my $newSetHeaderFile = stripWhiteSpace($inputs{'setHeaderFile'});
    my $newProbHeaderFile = stripWhiteSpace($inputs{'probHeaderFile'});

    my $time1 = &unformatDateAndTime($newOpenDate);
    my $time2 = &unformatDateAndTime($newDueDate);
    my $time3 = &unformatDateAndTime($newAnswerDate);
    if ($time2 < $time1 or $time3 < $time2) {
        &Global::error('Dates not in chronological order', "The open date: $newOpenDate,
        due date: $newDueDate, and answer date: $newAnswerDate must be in chronologicasl order.");
    }

    my $setNumber=$inputs{'setNo'};

    changeDatesAndHeaderFiles($setNumber,$newOpenDate,$newDueDate,$newAnswerDate,$newSetHeaderFile,$newProbHeaderFile);

#    &changeOpenDate($newOpenDate,$setNumber);
#    print "<BR><BR>\n";
#    &changeDueDate($newDueDate,$setNumber);
#    print "<BR><BR>\n";
#    &changeAnswerDate($newAnswerDate,$setNumber);
#    print "<BR><BR>\n";
#    &changeSetHeaderFile($newSetHeaderFile,$setNumber);
#    print "<BR><BR>\n";
#	&changeProbHeaderFile($newProbHeaderFile,$setNumber);
#    print "<BR><BR>\n";

}

sub findOpenDateForSet {
	my ($setNumber) = @_;
    my (@probSetKeys)=&getAllProbSetKeysForSet($setNumber);
    &attachProbSetRecord($probSetKeys[0]);
    &detachProbSetRecord($probSetKeys[0]);
    &formatDateAndTime( &getOpenDate($probSetKeys[0]));
}

sub printChangeOpenForm {
	my ( $prettyOldOpenDate) = @_;
    print <<EOF;
<H3 ALIGN ="CENTER">Class Identification: $inputs{'course'}</H3>
Use this form to makes changes for an entire set.  To make changes
for an individual student go back and use "Examine or change individual problem set for:"

<HR SIZE =2>
<H3> Change opening and closing dates for set number $setNumber</H3>
Please enter the new dates in the following format:<BR>6/28/96 at
9:59 AM (June 28, 1996 at 9:59 AM)<p>

Select the method you want to use. If e.g. you have given a few students an extension and
do not want to override those changes, select the second method. <p>
<INPUT TYPE=RADIO NAME='updateMethod' VALUE='all' > Change dates for all students.<BR>
<INPUT TYPE=RADIO NAME='updateMethod' VALUE='some' CHECKED>
Change dates only if the new date is later than an individual student\'s current date.<P>

The current openDate is
$prettyOldOpenDate

<input type="text" size=20 maxlength=20 name="openDate"
value="$prettyOldOpenDate"> New openDate<p>

EOF
}

sub findDueDateForSet {
    my (@probSetKeys)=&getAllProbSetKeysForSet($setNumber);
    &attachProbSetRecord($probSetKeys[0]);
    &detachProbSetRecord($probSetKeys[0]);
    &formatDateAndTime( &getDueDate($probSetKeys[0]));
}

sub printChangeDueForm {
	my ( $prettyOldDueDate) = @_;
    print <<EOF;


<p> The current DueDate is $prettyOldDueDate

<input type="text" size=20 maxlength=20 name="dueDate"
value="$prettyOldDueDate"> New DueDate<p>

EOF
}

sub findAnswerDateForSet {
    my (@probSetKeys)=&getAllProbSetKeysForSet($setNumber);
    &attachProbSetRecord($probSetKeys[0]);
    &detachProbSetRecord($probSetKeys[0]);
    &formatDateAndTime( &getAnswerDate($probSetKeys[0]));
}

sub printChangeAnswerForm {
    my ( $prettyOldAnswerDate) = @_;
    print <<EOF;


<p> The current AnswerDate is $prettyOldAnswerDate

<input type="text" size=20 maxlength=20 name="ansDate"
value="$prettyOldAnswerDate"> New AnswerDate<p>

EOF
}

sub changeDatesAndHeaderFiles {
	my ($setNumber,$newOpenDate,$newDueDate,$newAnswerDate,$newSetHeaderFileName,$newProbHeaderFileName) = @_;

    my $newOpenDateTime = &unformatDateAndTime($newOpenDate);
    my $newDueDateTime = &unformatDateAndTime($newDueDate);
    my $newAnswerDateTime = &unformatDateAndTime($newAnswerDate);

    my $oldOpenDateTime;
    my $oldDueDateTime;
    my $oldAnswerDateTime;

    my @probSetKeys=&getAllProbSetKeysForSet($setNumber);
    my $probSetKey;

    foreach $probSetKey (@probSetKeys) {
		&attachProbSetRecord($probSetKey);

		if ($updateMethod eq 'all') {
			&putOpenDate($newOpenDateTime,$probSetKey);
			&putDueDate($newDueDateTime,$probSetKey);
			&putAnswerDate($newAnswerDateTime,$probSetKey);
		}
		else {
			$oldOpenDateTime = &getOpenDate($probSetKey);
			&putOpenDate($newOpenDateTime,$probSetKey) unless ($oldOpenDateTime > $newOpenDateTime);

			$oldDueDateTime = &getDueDate($probSetKey);
			&putDueDate($newDueDateTime,$probSetKey) unless ($oldDueDateTime > $newDueDateTime);

			$oldAnswerDateTime = &getAnswerDate($probSetKey);
			&putAnswerDate($newAnswerDateTime,$probSetKey) unless ($oldAnswerDateTime > $newAnswerDateTime);
		}
		&putSetHeaderFileName( $newSetHeaderFileName,$probSetKey);
		&putProbHeaderFileName( $newProbHeaderFileName,$probSetKey);

		&detachProbSetRecord($probSetKey);
	}

    print "<BR><BR>The new open date is now set to: $newOpenDate  for all students\n";
    print "except if their original open date was later than $newOpenDate\n" unless ($updateMethod eq 'all');
    print "<HR><BR>\n";

    print "The new due date is now set to: $newDueDate for all students\n";
    print "except if their original due date was later than $newDueDate\n" unless ($updateMethod eq 'all');
    print "<HR><BR>\n";

    print "The new answer date is now set to: $newAnswerDate for all students\n";
    print "except if their original answer date was later than $newAnswerDate\n" unless ($updateMethod eq 'all');
    print "<HR><BR>\n";

    print "The new Paper HeaderFile is: $newSetHeaderFileName \n";
    print "<HR><BR>\n";

    print "The new Screen HeaderFile is: $newProbHeaderFileName \n";
    print "<HR><BR>\n";

}

#sub changeOpenDate {
#	my ($newOpenDate,$setNumber) = @_;
#    my $newOpenDateTime = &unformatDateAndTime($newOpenDate);
#    my @probSetKeys=&getAllProbSetKeysForSet($setNumber);
#    my $probSetKey;
#    my $oldOpenDateTime;
#    foreach $probSetKey (@probSetKeys) {
#		&attachProbSetRecord($probSetKey);
#		if ($updateMethod eq 'all') {&putOpenDate($newOpenDateTime,$probSetKey);}
#		else {
#			$oldOpenDateTime = &getOpenDate($probSetKey);
#			&putOpenDate($newOpenDateTime,$probSetKey) unless ($oldOpenDateTime > $newOpenDateTime);
#		}
#		&detachProbSetRecord($probSetKey);
#	}
#
#    print "The new open date is now set to: $newOpenDate  for all students\n";
#    print "except if their original open date was later than $newOpenDate\n" unless ($updateMethod eq 'all');
#    print "_"x80 . "\n";
#}
#
#sub changeDueDate {
#	my ($newDueDate,$setNumber) = @_;
#    my $newDueDateTime = &unformatDateAndTime($newDueDate);
#    my @probSetKeys=&getAllProbSetKeysForSet($setNumber);
#	my $probSetKey;
#	my $oldDueDateTime;
#    foreach $probSetKey (@probSetKeys) {
#		&attachProbSetRecord($probSetKey);
#		if ($updateMethod eq 'all') {&putDueDate($newDueDateTime,$probSetKey);}
#		else {
#			$oldDueDateTime = &getDueDate($probSetKey);
#			&putDueDate($newDueDateTime,$probSetKey) unless ($oldDueDateTime > $newDueDateTime);
#		}			&detachProbSetRecord($probSetKey);
#	    }
#
#    print "The new due date is now set to: $newDueDate for all students\n";
#    print "except if their original due date was later than $newDueDate\n" unless ($updateMethod eq 'all');
#    print "_"x80 . "\n";
#
#}
#
#sub changeAnswerDate {
#	my ($newAnswerDate,$setNumber) = @_;
#    my $newAnswerDateTime = &unformatDateAndTime($newAnswerDate);
#    my @probSetKeys=&getAllProbSetKeysForSet($setNumber);
#	my $probSetKey;
#	my $oldAnswerDateTime;
#    foreach $probSetKey (@probSetKeys){
#		&attachProbSetRecord($probSetKey);
#		if ($updateMethod eq 'all') {&putAnswerDate($newAnswerDateTime,$probSetKey);}
#		else {
#			$oldAnswerDateTime = &getAnswerDate($probSetKey);
#			&putAnswerDate($newAnswerDateTime,$probSetKey) unless ($oldAnswerDateTime > $newAnswerDateTime);
#		}			&detachProbSetRecord($probSetKey);
#	    }
#
#    print "The new answer date is now set to: $newAnswerDate for all students\n";
#    print "except if their original answer date was later than $newAnswerDate\n" unless ($updateMethod eq 'all');
#    print "_"x80 . "\n";
#	}


sub findSetHeaderFileForSet {
	my ($setNumber) = @_;
    my (@probSetKeys)=&getAllProbSetKeysForSet($setNumber);
    &attachProbSetRecord($probSetKeys[0]);
    my $setHeaderFileName = &getSetHeaderFileName($probSetKeys[0]);
	$setHeaderFileName;
	}
sub findProbHeaderFileForSet {
	my ($setNumber) = @_;
    my (@probSetKeys)=&getAllProbSetKeysForSet($setNumber);
    &attachProbSetRecord($probSetKeys[0]);
    my $probHeaderFileName = &getProbHeaderFileName($probSetKeys[0]);
	$probHeaderFileName;
	}
sub printChangeSetHeaderFileForm {
	my ( $oldSetHeaderFileName) = @_;
	$oldSetHeaderFileName = "" unless defined($oldSetHeaderFileName);
    print <<EOF;

<H3> Change the  Paper and Screen HeaderFiles for set number $setNumber</H3>
 The current Paper HeaderFile is $oldSetHeaderFileName

<input type="text" size=30 maxlength=50 name="setHeaderFile"
value="$oldSetHeaderFileName"> New SetHeaderFile<p>

EOF
}

sub printChangeProbHeaderFileForm {
	my ( $oldProbHeaderFileName) = @_;
	$oldProbHeaderFileName = "" unless defined $oldProbHeaderFileName;
    print <<EOF;

 The current Screen HeaderFile is $oldProbHeaderFileName

<input type="text" size=30 maxlength=50 name="probHeaderFile"
value="$oldProbHeaderFileName"> New ProbHeaderFile<p>

EOF
}

sub changeSetHeaderFile {
	my ($newSetHeaderFileName,$setNumber) = @_;
    my @probSetKeys=&getAllProbSetKeysForSet($setNumber);
	my $probSetKey;
    foreach $probSetKey (@probSetKeys){
		&attachProbSetRecord($probSetKey);
		&deleteSetHeaderFileName($probSetKey);
		&putSetHeaderFileName( $newSetHeaderFileName,$probSetKey);
		&detachProbSetRecord($probSetKey);
	    }
    &attachProbSetRecord($probSetKeys[0]);
    $newSetHeaderFileName = &getSetHeaderFileName($probSetKeys[0]);
    print "The new Paper HeaderFile is: $newSetHeaderFileName \n";
    print "_"x80 . "\n";
    &detachProbSetRecord($probSetKeys[0]);
	}
sub changeProbHeaderFile {
	my ($newProbHeaderFileName,$setNumber) = @_;
    my @probSetKeys=&getAllProbSetKeysForSet($setNumber);
	my $probSetKey;
    foreach $probSetKey (@probSetKeys){
		&attachProbSetRecord($probSetKey);
		&deleteProbHeaderFileName($probSetKey);
		&putProbHeaderFileName( $newProbHeaderFileName,$probSetKey);
		&detachProbSetRecord($probSetKey);
	    }
    &attachProbSetRecord($probSetKeys[0]);
    $newProbHeaderFileName = &getProbHeaderFileName($probSetKeys[0]);
    print "The new Screen HeaderFile is: $newProbHeaderFileName \n";
    print "_"x80 . "\n";
    &detachProbSetRecord($probSetKeys[0]);
	}



sub sendBack {
print <<EOF;
<p>
     <A HREF="${cgiURL}profLogin.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
          <IMG SRC="$Global::upImgUrl" BORDER=1></A>
EOF
}



sub printChangeProblemFileNamesForm {

	my $delete_cached_images = 0;	## if problems are deleted or reseeded, this will mess up
										## the chached latex2html image. We keep track of this so
										## we can delete them.

#   Update the database if needed
if ( ( defined( $inputs{'FileNameEditMode'}) && $inputs{'FileNameEditMode'} eq "ReadWrite" )
	 ||
	 ( defined( $inputs{'AddNewProblem'})   && $inputs{'AddNewProblem'}    eq "Add"     ) )
	{

#   get list of changed problems submitted by the form
    my $i;
    my @changedProblems;
   	foreach $i ( grep(/ProblemFileName/, keys %inputs) ) {

	    if ( $i =~ /^ProblemFileName(.+)$/ ) {
	    	push(@changedProblems, $1);
	    }
	    if ($inputs{'AddNewProblem'} eq "Add" ) {
	    	if ( $i =~ /^NewProblemFileName(.+)$/ ) {
	    	    my $label = $1;
	    	    # file name cannot be blank and cannot contain a question mark.
	    	    if ($inputs{"NewProblemFileName$label"} =~/\w+/ && not $inputs{"NewProblemFileName$label"} =~/\?/) {
			    	push(@changedProblems, $label) ;
			    	$inputs{"ProblemFileName$label"}=$inputs{"NewProblemFileName$label"};
			    	$inputs{"ProblemValue$label"}=$inputs{"NewProblemValue$label"};
			    }
			}
		}
	}

 ##############################
### Later this program assumes that the problem numbers in @changedProblems are in ascending order,
### but the validity of this assumption depends on whether the hash  %inputs
### preserves the order of the form variables; in particular, do the keys
### ProblemFileName1, ProblemFileName2, ..., ProblemFileName(N), NewProblemFileName(N+1), etc.,
### occur in that order in the array    keys %inputs    .
### On harper.ucs.indiana.edu, which is a Sun running Sun OS 5.7,
### the keys do not necessarily occur in increasing order of the problem number.
### For short hashes, they do.  But for long hashes, they usually do not, and that causes
### this program to fail.  So we must have the line below to insure
### that @changedProblems is ordered in ascending order.
#############################
     @changedProblems = sort {$a <=> $b} @changedProblems;
###############################


   my $psvn;
   my @psvn = &getAllProbSetKeysForSet($setNumber);
   srand(time);
   foreach $psvn (@psvn)  {
       &attachProbSetRecord($psvn);
       my $num =1;
#       srand(time);	## Move this out of the loop as per Zig's suggestion.
						## Otherwise many students get the same seed - AKP
      foreach $i (@changedProblems) {

			## chech if we should delete cached l2h images
			if  ( (defined( $inputs{"problem{$i}ReSeed"}) and ($inputs{"problem{$i}ReSeed"} eq "on")) or
                          (defined( $inputs{"problem{$i}Delete"}) and ($inputs{"problem{$i}Delete"} eq "on"))) {
			  	$delete_cached_images = 1;
			}

            #ReadWrite
                    #save seed (or define it)
               		my  $seed = &getProblemSeed($i,$psvn);
               		unless (defined($seed) ){
               			print STDERR "profChangeDates.pl: Seed for problem $i is not defined<BR>";
    		        	$seed = int( rand(5000) );
    		        	}

					#save
					my ($filename, $status, $attempted, $pvalue, $maxNumOfIncorrectAttempts);
	    		    $filename = &getProblemFileName(    $i,$psvn);
	    		    $attempted = &getProblemAttempted(   $i,$psvn);
					$status = &getProblemStatus(   $i,$psvn);
        		  	$pvalue = &getProblemValue(       $i,$psvn);
					$maxNumOfIncorrectAttempts = &getProblemMaxNumOfIncorrectAttemps(  $i,$psvn);

    		        # delete problem names and values
    		    	&deleteProblemFileName(    $i,$psvn);
    		    	&deleteProblemAttempted(   $i,$psvn);
					&deleteProblemStatus(   $i,$psvn);
        		  	&deleteProblemValue(       $i,$psvn);
					&deleteProblemMaxNumOfIncorrectAttemps(  $i,$psvn);
        		  	&deleteProblemSeed(        $i,$psvn);
        		  	# recreate problem using saved seed
        		  	unless (defined( $inputs{"problem{$i}Delete"}) && $inputs{"problem{$i}Delete"} eq "on" ) {

						if ($inputs{"ProblemFileName$i"} =~ /\w+/) {$filename = stripWhiteSpace($inputs{"ProblemFileName$i"});}
        		  	 	&putProblemFileName($filename ,   $num,$psvn);

        		  	 	if ($inputs{"ProblemAttempted$i"} =~ /\w+/) {$attempted = stripWhiteSpace($inputs{"ProblemAttempted$i"});}
						&putProblemAttempted($attempted,     $num,$psvn);

						if ($inputs{"ProblemStatus$i"} =~ /\w+/) {$status = stripWhiteSpace($inputs{"ProblemStatus$i"});}
						&putProblemStatus($status,     $num,$psvn);

						if ($inputs{"ProblemValue$i"} =~ /\w+/) {$pvalue = stripWhiteSpace($inputs{"ProblemValue$i"});}
	        		  	&putProblemValue($pvalue,  $num,$psvn);

						if ($inputs{"ProblemMaxNumOfIncorrectAttemps$i"} =~ /\w+/) {$maxNumOfIncorrectAttempts = stripWhiteSpace($inputs{"ProblemMaxNumOfIncorrectAttemps$i"});}
						&putProblemMaxNumOfIncorrectAttemps($maxNumOfIncorrectAttempts,  $num,$psvn);

						&putProblemSeed(        $seed,$num,$psvn);

	        		  	# Reseed if requested
        		  		if  (( defined( $inputs{"problem{$i}ReSeed"}) && $inputs{"problem{$i}ReSeed"} eq "on")
                                        # But not if we are deleting
                           and not ( defined( $inputs{"problem{$i}Delete"}) && $inputs{"problem{$i}Delete"} eq "on")) {
        	    	      $seed = int( rand(5000) );
        	    	      &putProblemSeed( $seed ,  $num,$psvn);
        	    	      print "Seed changed for problem number $num, psvn $psvn<BR>\n";
        	    	      }
        	    	    $num++;
        		  		}


        	   		}
        &detachProbSetRecord($psvn);
        }
} # end of updating database

## remove l2h images if necessary

if ($delete_cached_images) {
	my $l2hDir = getCoursel2hDirectory();
	system ("rm -rf ${l2hDir}set$setNumber");
	print "\n<BR>deleting Latex2html tmp files and the directory:<BR>\n";
	print "     ${l2hDir}set$setNumber<BR>";
	# Now delete dvipng images
	$l2hDir = getCourseTempDirectory()."png/";
	system ("rm -rf ${l2hDir}$setNumber");
	print "\n<BR>deleting dvipng tmp files and the directory:<BR>\n";
	print "     ${l2hDir}$setNumber<BR>\n";
}


# print a new form
    print qq!<HR NOSHADE>\n
	 	     <FORM method="post" action="profChangeDates.pl">
	 	     !;
	print     &sessionKeyInputs(\%inputs);
	if (defined( $inputs{"FileNameEditMode"}) && $inputs{"FileNameEditMode"} eq "ReadWrite") {
		print     "CHANGES SAVED FOR SET $setNumber<BR>";
	} elsif ( defined( $inputs{'AddNewProblem'})   && $inputs{'AddNewProblem'}    eq "Add"     )   {
		print     "Problems added to SET $setNumber<BR>";
	} elsif ( defined( $inputs{"FileNameEditMode"}) && $inputs{"FileNameEditMode"} eq "ReadOnly") {
		print     "ReadOnly mode -- <B>NO</B> CHANGES SAVED FOR SET $setNumber<BR>";
	}
	my @psvn = &getAllProbSetKeysForSet($setNumber);

	print qq!
	        <INPUT TYPE="HIDDEN" NAME="pCD" VALUE="1">\n
			<INPUT TYPE="HIDDEN" NAME="setNo" VALUE="$setNumber">\n

			<H3>Change problem file names, status, etc. for set $setNumber:</H3>\n

			<H4>If a Filename, "Attempted", "Status", "Value", or "MaxAttmp" cell is left blank, the corresponding
			individualized current values in the database will be maintained. If a value is
			entered, that value will be used for every student's problem. Valid values for "Attempted"
			are: 0 (not attempted) or 1. Valid values for "Status" are numbers in the range [0,1] where 0 represents
			no credit and 1 full crefit.  Valid values for "Value" are
			non-negative integers with 1 (1 point) being the most common.  Valid values for "MaxAttmp"
			are -1 and non-negative integers where -1 means allow unlimited attempts.  The Filename
			for a random student is listed. To see the actual data for any student, go back and use
			"Examine or change individual problem set for:"
			</H4>\n

			<TABLE><TR><TD>\n
			 ReadOnly<INPUT TYPE=RADIO NAME="FileNameEditMode" VALUE="ReadOnly" CHECKED>\n
			 </TD><TD>\n
			 Read/Write<INPUT TYPE=RADIO NAME="FileNameEditMode" VALUE="ReadWrite" >\n
			 </TD><TD>\n
	         <INPUT TYPE=SUBMIT VALUE="Save Problem Changes"><p>\n
	         </TD></TR></TABLE>
	         <TABLE BORDER="1" CELLPADDING="1" CELLSPACING="2" >\n
			 !;
#Re-establish the order of problems from database -- they may have changed.
    my $psvn = $psvn[0];
    my $Mode = defined($inputs{'mode'}) ? $inputs{'mode'} : "HTML";
    my $User = $inputs{'user'};
    my $Key =  $inputs{'key'};
	&attachProbSetRecord($psvn);
	my @problems = sort {$a <=> $b} &getAllProblemsForProbSetRecord($psvn);


# print table listing problems
    my $i;
	foreach $i (@problems) {
	   	print "<TR>";
	        print &formatHeaderCell( qq!
	            <A HREF="${Global::processProblem_CGI}?probSetKey=$psvn&probNum=$i&Mode=$Mode&course=$Course&user=$User&key=$Key" TARGET="VIEW_PROBLEM">
	           Problem $i</A>
	           !
	           );
	           print &formatHeaderCell( "Attempted");
			   print &formatHeaderCell( "Status");
	           print &formatHeaderCell( "Value");
			   print &formatHeaderCell( "MaxAttmp");
	           print &formatHeaderCell( "Delete" );
	           print &formatHeaderCell( "ReSeed" );
	           print "</TR>";


	    print "<TR>";

	        print &formatDataCell( "ProblemFileName$i"  , &getProblemFileName($i,$psvn),"50" );
	        print &formatDataCell( "ProblemAttempted$i"    , ''   ,"1");
			print &formatDataCell( "ProblemStatus$i"    ,'' ,"5");
	        print &formatDataCell( "ProblemValue$i"     ,''    ,"5");
			print &formatDataCell( "ProblemMaxNumOfIncorrectAttemps$i"     , ''    ,"4");
	        print & formatCheckboxButtonCell( "problem{$i}Delete", "5", "");
			print & formatCheckboxButtonCell( "problem{$i}ReSeed", "5", "");

	    print "</TR>";

    	}

	print qq!
		</TABLE>\n
		<HR NOSHADE>
		<H3>Add problem file names for set $setNumber</H3>\n
		Off <INPUT TYPE="RADIO" NAME="AddNewProblem" VALUE="Off" CHECKED> \n
		Add <INPUT TYPE="RADIO" NAME="AddNewProblem" VALUE="Add"> \n
		<INPUT TYPE=SUBMIT VALUE="Add File Names to Set"><p>\n
		<TABLE BORDER="1" CELLPADDING="1" CELLSPACING="2" >\n
		!;
# print table new listing problems
	foreach $i (101,102) {
	    print "<TR>";
	           print &formatHeaderCell( "New File $i");
	           print &formatHeaderCell( "Attempted");
	           print &formatHeaderCell( "Status");
			   print &formatHeaderCell( "Value");
			   print &formatHeaderCell( "MaxAttmp");
	           print "</TR>\n";


	    print "<TR>";

	        print &formatDataCell( "NewProblemFileName$i"  , " " ,"30" );
	        print &formatDataCell( "NewProblemAttempted$i"    ,  "0"  ,"1");
			print &formatDataCell( "NewProblemStatus$i"    ,  "0"  ,"5");
	        print &formatDataCell( "NewProblemValue$i"     , "1" ,"5");
			print &formatDataCell( "ProblemMaxNumOfIncorrectAttemps$i"     ,  "-1"    ,"4");

	    print "</TR>\n";
	    }
print qq!
  	</TABLE>\n

  	</FORM>\n
  	!;
#############################################
# build a problem set
#############################################

print <<EOF;
<P>
<HR NOSHADE>
<H4 ALIGN=LEFT>
<img src="$Global::bluesquareImgUrl" border=1 alt="">
 Build a problem set for $Course:
 </H4>
EOF

print "<TD>",
start_form('POST', "${Global::cgiWebworkURL}profBuildProblemSetPage.pl"),
p,"\n",
hidden('course'), "\n",
hidden('user'),   "\n",
hidden('key'),    "\n",
submit("Enter Build Problem Set Page"),
end_form,
"</TD>",
"You can build a problem set, edit a set definition file, etc.";

# delete an entire problem set
print <<EOF;
<P>
<HR NOSHADE>
<H4 ALIGN=LEFT><A HREF="${cgiURL}profDeleteProbSet.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
         <IMG SRC="$Global::bluesquareImgUrl"
              BORDER=1
              ALT="">  Delete a problem set in $Course</A>
</H4>
  Remove an entire problem set from the database.
              Individual problems can be deleted from the set using the method 2. above.

EOF

	} #DONE printChangeProblemFileNamesForm()

#sub printAddProblemFileNameForm {
#
#	my @problems = ("new1","new2");
#
## updating will be done in printChangeProblemFileNamesForm()
#
## print a new form
#    print qq!<HR NOSHADE>\n
#	 	     <FORM method="post" action="profChangeDates.pl">
#	 	     !;
#	print     &sessionKeyInputs(\%inputs);
#	print qq!
#	        <INPUT TYPE="HIDDEN" NAME="pCD" VALUE="1">\n
#			<INPUT TYPE="HIDDEN" NAME="setNumber" VALUE="$setNumber">\n
#			<HR NOSHADE>
#			<H3>Add problem file names for set $setNumber</H3>\n
#			Off <INPUT TYPE="RADIO" NAME="AddNewProblem" VALUE="Off"><BR>\n
#			Add <INPUT TYPE="RADIO" NAME="AddNewProblem" VALUE="Add"><BR>\n
#			<INPUT TYPE=SUBMIT VALUE="Add New Problem"><p>\n
#			<TABLE BORDER="1" CELLPADDING="1" CELLSPACING="2" >\n
#			!;
## print table new listing problems
#	foreach $i ("9998") {
#	    print "<TR>";
#	           print &formatHeaderCell( $i);
#	           print &formatHeaderCell( "Value"  );
#	           print "</TR>\n";
#
#
#	    print "<TR>";
#
#	        print &formatDataCell( "NewProblemFileName$i"  , "???" ,"30" );
#	        print &formatDataCell( "NewProblemValue$i"     , "???" ,"5");
#
#	    print "</TR>\n";
#	    }
#print qq!
#  	</TABLE>\n
#
#  	</FORM>\n
#  	!;
#  }

sub formatDataRow {
    my ($pAitems,$size) =@_;
    my $out = "<TR>\n";
    my $value;
    my $i;
    foreach $i (@$pAitems) {
        $value = eval "\$$i=&get$i($inputs{'psvn'})";
        $out .= &formatDataCell($i,$value,$size);
        }
    $out .= "</TR>\n";
    }
sub formatHeaderRow {
    my ( $pAitems,$options) =@_;
    my $out = "<TR>\n";
    my $i;
    foreach $i ( @$pAitems) {
        $out .= &formatHeaderCell($i,$options);
        }
    $out .= "</TR>\n";
    }
sub formatDataCell {
    my	($name,$value,$size) = @_;
    $value = "" unless defined($value);	# this value is optional
    $size = "" unless defined($size);  		# this value is optional
    my $out = qq!
    <TD ALIGN=CENTER VALIGN=MIDDLE >
    <INPUT TYPE="TEXT" NAME="$name" VALUE="$value", SIZE=$size>
    </TD>
    !;
    $out;
    }
sub formatCheckboxButtonCell {
    my ($name,$size, $options) = @_;
    $options = "" unless defined($options);
    my $out = qq!
    <TD ALIGN=CENTER VALIGN=MIDDLE >
    <INPUT TYPE="CHECKBOX" NAME="$name" VALUE="on" SIZE="$size" $options>
    </TD>
    !;
    $out;
    }
sub formatHeaderCell {
    my ($item,$options) = @_;
    $options = "" unless defined($options);
    my $out = qq!
    <TH ALIGN=CENTER VALIGN=MIDDLE $options>
    $item
    </TH>
    !;

    }
