#!/usr/local/bin/webwork-perl

## This file is profFormEditScoringFiles.pl
## It provides access to utilities for downloading scoring files
##

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################

use lib '.'; use webworkInit; # WeBWorKInitLine
use CGI qw(:standard);
use Global;
use Auth;
use strict;
use File::Copy;

# begin Timing code
use Benchmark;
my $beginTime = new Benchmark;
# end Timing code

&CGI::ReadParse;
my %inputs =%main::in;

# get information from CGI inputs  (see also below for additional information)

my $Course 			= $inputs{'course'};
my $User 			= $inputs{'user'};
my $Session_key 	= $inputs{'key'};

my @array = %inputs;
# print "inputs are @array";

# verify that information has been received
unless($Course && $User && $Session_key) {
    &wwerror("$0","The script did not receive the proper input data.","","");
}


# establish environment for this script

&Global::getCourseEnvironment($inputs{'course'});

my $cgiURL   				= getWebworkCgiURL;
my $databaseDirectory 		= getCourseDatabaseDirectory;
my $htmlURL       			= getCourseHtmlURL;
my $scriptDirectory   		= getWebworkScriptDirectory;
my $scoringDirectory        = getCourseScoringDirectory;
my $DAT                     = getDat;
my $dd                      = getDirDelim;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";
require "${scriptDirectory}$Global::SCRtools_pl";

# log access
&Global::log_info('', query_string);


my $permissionsFile = &Global::getCoursePermissionsFile($inputs{'course'});
my $permissions = &get_permissions($inputs{'user'}, $permissionsFile);
my $keyFile = &Global::getCourseKeyFile($inputs{'course'});

#verify session key
&verify_key($inputs{'user'}, $inputs{'key'}, $keyFile, $inputs{'course'});

# verify permissions are correct
if ($permissions != $Global::instructor_permissions ) {
    print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
    print &html_NO_PERMISSION;
    exit(0);
}
# get the rest of the information from the submitted form


my $totalsFile      = $inputs{'totalsFile'};
my $scrFiles        = $inputs{'scrFiles'};
my $fulFiles        = $inputs{'fulFiles'};
my $fileName        = $inputs{'fileName'};
my $action          = $inputs{'action'};
my $readWrite       = $inputs{'readWrite'};

## define values even if check boxes not checked
$totalsFile = 0 unless defined $totalsFile;
$scrFiles = 0 unless defined $scrFiles;
$fulFiles = 0 unless defined $fulFiles;

## if totals file is the only one selected, view it immediately

OPTIONSWITCH: {
    if ($totalsFile == 1  and  $scrFiles == 0  and  $fulFiles == 0 and $action eq '') {
        $fileName = "${scoringDirectory}${Course}_totals.csv";
        unless (-r $fileName) {
            &wwerror("$0","The file $fileName is not readable by the webserver","","");
        }
        &createFormFromFile($fileName);
       last  OPTIONSWITCH;
    }
    if ($scrFiles > 0  or  $fulFiles > 0 and $action eq '') {
        &selectFilesForm();
        last  OPTIONSWITCH;
    }
    if ($action eq 'readFromFile') {
        &createFormFromFile("${scoringDirectory}$fileName");
        last  OPTIONSWITCH;
    }
    if ($action eq 'saveToFile') {
        &saveToFile(\%inputs);
        last  OPTIONSWITCH;
    }
    if ($action eq 'addEmptyColumn') {
        &addColumn(\%inputs);
        last  OPTIONSWITCH;
    }
    if ($action eq 'deleteLastColumn') {
        &deleteLastColumn(\%inputs);
        last  OPTIONSWITCH;
    }        
    if ($totalsFile == 0  and  $scrFiles == 0  and  $fulFiles == 0 and $action eq '') {
       &wwerror("$0","You must check at least one check box","","");
       last  OPTIONSWITCH;
    }
}

# begin Timing code
my $endTime = new Benchmark;
&Global::logTimingInfo($beginTime,$endTime,"profScoring.pl",$inputs{'course'},$inputs{'user'});
# end Timing code

exit;

## subroutines


sub selectFilesForm {

# print HTML text
print &htmlTOP("Select Scoring File for simple editing");

# print navigation buttons
print qq!
<A HREF="${cgiURL}profLogin.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
!;

print <<EOF;
<h3 align="left">Simple editing of scoring files for $Course</h3> <p>
EOF


print <<EOF;
<P>
<HR NOSHADE>
<H4 ALIGN=LEFT>
Select files:
</H4>

		<FORM METHOD = "POST" ACTION= "${cgiURL}profFormEditScoringFiles.pl">
         <INPUT TYPE="SUBMIT" VALUE="Edit file">

EOF

 ## find the available files

opendir SCORINGDIR, $scoringDirectory or wwerror("$0","Can't open directory $scoringDirectory","","");
my @allFiles = grep !/^\./, readdir SCORINGDIR;
closedir  SCORINGDIR;

    ## sort the files

my @totalsFiles = grep /${Course}_totals\.csv$/,@allFiles;
my @scrFiles =  grep /scr\.csv$/,@allFiles;
my @fulFiles =  grep /ful\.csv$/,@allFiles;
my  @sortedNames = ();

if ($totalsFile) {
    @totalsFiles = sort @totalsFiles;
    @sortedNames = (@sortedNames,@totalsFiles);
}
if ($scrFiles) {
    @scrFiles = sort @scrFiles;
    @sortedNames = (@sortedNames, @scrFiles);
}
if ($fulFiles) {
    @fulFiles = sort @fulFiles;
    @sortedNames = (@sortedNames, @fulFiles);
}

## print list of files

print qq! <SELECT Name="fileName" >\n!;
my ($ind);
for $ind (@sortedNames) {
	print "<OPTION VALUE = \"$ind\">$ind\n";
}
    print qq!</SELECT>\n!;

 ## resume printing the rest of the form

print <<EOF;
         <INPUT TYPE="HIDDEN" NAME="user" VALUE="$inputs{'user'}">
         <INPUT TYPE="HIDDEN" NAME="key" VALUE="$inputs{'key'}">
         <INPUT TYPE="HIDDEN" NAME="course" VALUE="$inputs{'course'}">
         <INPUT TYPE="HIDDEN" NAME="action" VALUE="readFromFile">
         </FORM>
EOF
print &htmlBOTTOM("profFormEditScoringFiles.pl", \%inputs);
}



sub selectionError {
  print"content-type:\n\n<H2>Error:  You must make a selection!</H2>\n";
  print "<FORM METHOD=POST ACTION=\"${cgiURL}profFormEditScoringFiles.pl\"><P>";
  print &sessionKeyInputs(\%inputs);
print <<"ENDOFHTML";
<INPUT TYPE=SUBMIT VALUE="Return to Form Edit Scoring Files Page">
</FORM>
ENDOFHTML
  print &htmlBOTTOM("profFormEditScoringFiles.pl", \%inputs);
}

sub createFormFromFile {
    my ($fileName) = @_;
    &printTopOfForm();
	my $string = &delim2html($fileName,'htmlform');
    print "$string";
    &printBottomOfForm();
}


sub saveToFile {
    my ($inputsref) = @_;
    my $outString = &htmlForm2delim($inputsref);
#print "$outString";

#print "readWrite is $readWrite";

    unless ($readWrite eq 'readWrite') {
        &readWriteError();
        exit;
    }

    my $fulFileName     =   $inputs{'fileName'};
    my $shortFileName   =   $fulFileName;
    if ($shortFileName  =~  m|$dd|)  {
        $shortFileName  =~  m|$dd([^$dd]*)$|;  ## extract filename from full path name
        $shortFileName  =   $1;
    }    
    $shortFileName      =~  s|\..*||;      ## remove extension
        
    ## check for validity of format before saving file    
    my $i=1;
    while(-e "${scoringDirectory}temp$i.$DAT") {$i++;}      ##don't overwrite existing files
	my $tempFileName ="${scoringDirectory}temp$i.$DAT";
    open(SCORES,">$tempFileName")   or
      &wwerror("$0","Can't open $tempFileName","","", query_string());
    print SCORES "$outString";
    close SCORES;	
	my $shortTempFileName = "temp$i";	
    my $msg = htmlcheckdat($shortTempFileName);
            ## remove temp file
    unlink $tempFileName or &wwerror($0, "Could not remove the file $shortTempFileName");
    unless ($msg eq 'OK') {
        $msg .= "<BR>GO BACK AND CORRECT YOUR FORM. <BR>FILE NOT SAVED.";
        &wwerror("$0","$msg","","", query_string());
    }    	
    ## now that we know the file has a valid format, we can save it
	
    &backup("$shortFileName");   ##make up to three backups
    my $gid = $Global::numericalGroupID;
    $gid = $Global::numericalGroupID;    ## hack to remove warning message about $Global::numericalGroupID used only once
    
    my $tmpFileName = "${fulFileName}.tmp";
    &createFile($tmpFileName, 0660, $gid);
    open(SCORES,">$tmpFileName")   or
      &wwerror("$0","Can't open $tmpFileName");
    print SCORES "$outString";
    close SCORES;
    move("$tmpFileName","$fulFileName") or
      &wwerror("$0","Can't save $fulFileName");
    &readWriteSuccess($fulFileName);
}

sub readWriteError {
    print"content-type:\n\n<H2>No changes made:  You are in read only mode</H2>\n";
    print "Use your browser's BacK Button to return to the previous page or<BR>";
    print "<FORM METHOD=POST ACTION=\"${cgiURL}profScoring.pl\"><P>";
    print &sessionKeyInputs(\%inputs);
    print <<"ENDOFHTML";
    <INPUT TYPE=SUBMIT VALUE="Return to Scoring Page">
    </FORM>
ENDOFHTML
    print &htmlBOTTOM("profFormEditScoringFiles.pl", \%inputs);
}

sub readWriteSuccess {
  my ($savedFile) = @_;
  print"content-type:\n\n<H2>Success, file saved. </H2>\n";
  print "The contentsof your page have been saved in the file:<BR>$savedFile";
    
  print "<FORM METHOD=POST ACTION=\"${cgiURL}profScoring.pl\"><P>";
  print &sessionKeyInputs(\%inputs);
print <<"ENDOFHTML";
<INPUT TYPE=SUBMIT VALUE="Return to Scoring Page">
</FORM>
ENDOFHTML
  print &htmlBOTTOM("profFormEditScoringFiles.pl", \%inputs);
}

sub addColumn {
    my ($inputsref) = @_;
    my $inString = &htmlForm2delim($inputsref);
#print "$inString";
	my @inArray = split("\n",$inString);
	my $row;
	my @outArray =();
	foreach $row (@inArray) {
		$row .= ', ';
		push @outArray, $row;
	}				
    &createFormFromDelimitedArray(\@outArray,$fileName);
    exit;
}


sub deleteLastColumn {
    my ($inputsref) = @_;
    my $inString = &htmlForm2delim($inputsref);
#print "$inString";
	my @inArray = split("\n",$inString);
	my $row;
	my @outArray =();
	foreach $row (@inArray) {
		$row =~ s|,[^,]*$||;
		push @outArray, $row;
	}				
    &createFormFromDelimitedArray(\@outArray,$fileName);
    exit;
}

sub createFormFromDelimitedArray {
    my ($inArrayref,$fileName) = @_;
    &printTopOfForm();

    my $shortFileName = $fileName;                                                                                                                                                                                                                                                                                                                                                                                                                                    
    if ($shortFileName =~ m|$dd|)  {                                                                                                                                                                                                                                   
        $shortFileName =~ m|$dd([^$dd]*)$|;  ## extract filename from full path name                                                                                                                                                                                   
        $shortFileName = $1;                                                                                                                                                                                                                                           
    }                                                                                                                                                                                                                                                                  
    $shortFileName =~ s|\..*||;      ## remove extension   
	my $string = &delimitedArray2html($inArrayref, $shortFileName,'htmlform');
    print "$string";
    &printBottomOfForm();
}

sub printTopOfForm {
    print "content-type: text/html \n\n" ;
    
    # print navigation buttons
    print qq!
    <A HREF="${cgiURL}profScoring.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
    <IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
    !;
    
    print <<EOF;
		<FORM METHOD = "POST" ACTION= "${cgiURL}profFormEditScoringFiles.pl">
		<INPUT TYPE="SUBMIT" NAME = "action" VALUE="saveToFile">
		<INPUT TYPE="SUBMIT" NAME = "action" VALUE="addEmptyColumn">
		<INPUT TYPE="SUBMIT" NAME = "action" VALUE="deleteLastColumn">
		<INPUT TYPE="RESET" VALUE = "Reset Form"> <BR>
        <INPUT TYPE="radio" CHECKED NAME="readWrite" VALUE="readOnly" > Read Only Mode for saveToFile <BR>
        <INPUT TYPE="radio" NAME="readWrite" VALUE="readWrite"> Read/Write Mode for saveToFile<BR> 
        <strong>Changes are not saved until the "saveToFile" button is pressed and you see the message "Success, file saved".<BR>
        Three backups of your file will be kept, but please be careful.</strong><BR>

EOF
}

sub printBottomOfForm {
   print <<EOF;
         <INPUT TYPE="HIDDEN" NAME="user" VALUE="$inputs{'user'}">
         <INPUT TYPE="HIDDEN" NAME="key" VALUE="$inputs{'key'}">
         <INPUT TYPE="HIDDEN" NAME="course" VALUE="$inputs{'course'}">
         <INPUT TYPE="HIDDEN" NAME="fileName" VALUE="$fileName">
         </FORM>
EOF
print &htmlBOTTOM("profFormEditScoringFiles.pl", \%inputs);
}