#!/usr/local/bin/webwork-perl

## $Id: profLogin.pl,v 1.7 2002/06/10 19:18:09 apizer Exp $

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################


## This file is profLogin.pl
## It provides access to utilities for building, correcting and viewing problem sets
##
## It is called from a form with inputs
## 'user',
##^'key'
## 'course'

use lib '.'; use webworkInit; # WeBWorKInitLine
use CGI qw(:standard);
use Global;
use Auth;
use strict;

my $logTimingData = 0;
my $beginTime;

# begin Timing code
if ($logTimingData == 1) {
	use Benchmark;
	$beginTime = new Benchmark;
}
# end Timing code


my $cgi = new CGI;
my %inputs = $cgi->Vars();

# get information from CGI inputs  (see also below for additional information)

	my $Course 		= $cgi->param('course');
	my $User 		= $cgi->param('user');
	my $Session_key 	= $cgi->param('key');

# verify that information has been received
	unless($Course && $User && $Session_key) {
		&Global::error("Script Error","The script profLogin.pl did not receive the proper input data.","","");
		die "The script profLogin.pl did not receive the proper input data.";
		}

# establish environment for this script

	&Global::getCourseEnvironment($Course);


	my $cgiURL   			= $Global::cgiWebworkURL;
	my $courseScriptsDirectory 	= $Global::courseScriptsDirectory;
	my $databaseDirectory 		= $Global::databaseDirectory;
	my $htmlURL       		= $Global::htmlURL;
	my $scriptDirectory   		= $Global::scriptDirectory;

	require "${scriptDirectory}$Global::DBglue_pl";
	require "${scriptDirectory}$Global::classlist_DBglue_pl";
	require "${scriptDirectory}$Global::FILE_pl";
	require "${scriptDirectory}$Global::HTMLglue_pl";

# log access
	&Global::log_info('', query_string);


	my $permissionsFile = &Global::getCoursePermissionsFile($Course);
	my $permissions = &get_permissions($User, $permissionsFile);
	my $keyFile = &Global::getCourseKeyFile($Course);

#verify session key
	&verify_key($User, $Session_key, "$keyFile", $Course);

# verify permissions are correct
	if ($permissions != $Global::instructor_permissions ) {
		print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
		print &html_NO_PERMISSION;
		exit(0);
		}
# get the rest of the information from the submitted form
	# nothing further to get in this case

	my %availableSets = ();
	my @sortedSetNames = ();
	my @availableStudents = ();

# get information from the data base and create database if necessary

if ( -e "${databaseDirectory}$Global::database" ) {
	# a list of all of the currently created sets:
	# Keys contains the set numbers and the value is a representative psvn
	%availableSets = &getAllProbSetNumbersHash;
	@sortedSetNames = &sortSetNamesByDueDate(\%availableSets);
	@availableStudents = @{getAllLoginNamesSortedByName()};
}

# print HTML text
print &htmlTOP("WeBWorK Utilities");

# print navigation buttons

print $cgi->a( { -href=>"${cgiURL}login.pl?user=$User&key=$Session_key&course=$Course" },

			$cgi->img({ #  -name=>'upImg', # name is not a legal attribute for images
					  -src=>"${Global::upImgUrl}",
					  -align=>'right',
					  -border=>'1',
					  -alt=>'[Up]'
					})
			),
	$cgi->p;

print "\n",
	 $cgi->hr, $cgi->br,
	 "\n\n", $cgi->h3({ -align=>'left' }, "WeBWorK Utility programs for $Course"), "\n",
	 $cgi->p,
	 "From this page, you can monitor the progress of students and view their problems, build problem sets,
	 email students, score sets, edit the classlist database, reset passwords , change properties of built problem sets either for an individual student
	 or for all students, delete a problem set from the database and perform other tasks.",
	 $cgi->hr,
	 $cgi->center(
		$cgi->a({-href=>"#Student"}, "Student Progress"), " | ",
		$cgi->a({-href=>"#BuildProbSet"}, "Build Problem Set"), " | ",
	 	$cgi->a({-href=>"#Email"}, "Send Email"), " | ",
		$cgi->a({-href=>"#Score"}, "Scoring Page"), " | ",
		$cgi->a({-href=>"#AddStudents"}, "Add Student"), " | ",
		$cgi->a({-href=>"#profClasslist"}, "Classlist"), " | ",
		$cgi->a({-href=>"#dataMungerStudent"}, "Change Due Date"), " | ",
		$cgi->a({-href=>"#ChangeDates"}, "Edit Set"), " | ",
		$cgi->a({-href=>"#Housekeep"}, "Housekeeping"),
	 );

my %sethash; #popup_menu takes a hash for the labels to associate each label with a specific value
my %labels = ();

if ( -e "${databaseDirectory}$Global::database" ) {
### show only build problem set part if data base does not exist

# course data form
print heading('Section', "1. View statistical data on problem sets for the whole course and sections in $Course"),
	$cgi->startform(-action=>"${cgiURL}profCourseData.pl"),
	$cgi->submit(-value=>'View Course and Section Data'), "\n";


foreach (@sortedSetNames) {
	$sethash{$_} = "Set $_"; #insert the word Set before each set name
}

print $cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash), "\n",
	hiddens('user', 'key', 'course'), "\n",
	$cgi->endform(), "\n",
	"This allows you to see the average score on a set and on individual problems for
	the course as a whole and also by section.  The average number of incorrect
	attempts per problem is also displayed.";

# course recitation data form
print heading('Recitation', "2. View statistical data on problem sets for the whole course and recitations in $Course"),
	$cgi->startform(-action=>"${cgiURL}profCourseRecData.pl"),
	$cgi->submit(-value=>'View Course and Recitation Data'), "\n";



print $cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	"This allows you to see the average score on a set and on individual problems for
	the course as a whole and also by recitation.  The average number of incorrect
	attempts per problem is also displayed.";



# classlist.pl form -- view set data for the entire class

print heading('Student', "3. View students progress on problem sets in $Course:"),
	$cgi->startform(-action=>"${cgiURL}classlist.pl"),
	$cgi->submit(-value=>'View Students Progress'), "\n",
	$cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash), "\n",
	$cgi->br, "\n",
	"Order students \n",
	$cgi->radio_group(-name=>'sortOrder', -values=>['byCourse','bySection','byRecitation'], -labels=>{ 'byCourse'=>'Alphabetically', 'bySection'=>'by section ', 'byRecitation'=>'by recitattion '}),
	hiddens('user', 'key', 'course'),
	$cgi->hidden(-name=>'pCl', -value=>1), "\n",
	$cgi->endform(), "\n",
	"This also allows you to find the psvn for a given student and to view
	the problems as viewed by the student.  You can see the exact answer expected
	by WeBWorK and also, if you wish, answer the problem for the student.";


} ### end of if ( -e "${databaseDirectory}$Global::database" )
### show only build problem set part if data base does not exist

##  build problem set page form
print heading('BuildProbSet', "4. Build problem sets for $Course:"),
	$cgi->startform(-action=>"${cgiURL}profBuildProblemSetPage.pl"),
	$cgi->submit(-value=>'Enter Build Problem Set Page'), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	$cgi->p, "\n",
	"You can build problem sets, edit set definition files, etc.";


if ( -e "${databaseDirectory}$Global::database" ) {
### show only build problem set part if data base does not exist


#send email to students
print heading('Email', "5. Send Email to Students for $Course:"),
	$cgi->startform(-action=>"${cgiURL}profSendMail.pl"),
	$cgi->submit(-value=>'Enter Send Mail Page'), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	$cgi->p, "\n",
	"You can create and edit emails and send them to individual students or entire classlist
	at one time.  Macros can be used to insert student specific information into each students
	email on the fly.";

##  scoring page form
print heading('Score', "6. Scoring procedures for $Course:"),
	$cgi->startform(-action=>"${cgiURL}profScoring.pl"),
	$cgi->submit(-value=>'Enter Scoring Page'), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	$cgi->p, "\n",
	"You can score problem sets, download and upload spreadsheet scoring files, etc.";

##  add student form
print heading('AddStudents', "7. Add Student(s) to $Course:"),
	$cgi->startform(-action=>"${cgiURL}profAddStudent.pl"),
	$cgi->submit(-value=>'Enter Add Student(s) Page'), "\n",
	$cgi->hidden(-name=>'number', -value=>'5'), "\n",
	$cgi->hidden(-name=>'Action', -value=>'return'), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	$cgi->p, "\n",
	"You can add students to $Course.  This is usually used to add students who enter the course late.";

##  classlist page form

$labels{alph} = 'Alphabetically';
$labels{section} = 'by section';
$labels{recitation} = 'by recitation';

print heading('profClasslist', "8. Edit classlist database for $Course:"),
	$cgi->startform(-action=>"${cgiURL}profClasslist.pl"),
	$cgi->submit(-value=>'Enter Classlist Page'), "\n",
	$cgi->br, "\n",
	"Order students \n",
	$cgi->radio_group(-name=>'format', -values=>['alph','section','recitation'], -default=>'alph', -labels=>\%labels),
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	$cgi->p, "\n",
	"You can edit, export and import the classlist database and also reset passwords.";





##dataMunger form -- allows you to modify data for individualized set
## psvn is derived from the student name and the set number

print heading('dataMungerStudent', "9. Examine or modify data for an individual student in $Course:"),
	 $cgi->startform(-action=>"${cgiURL}dataMunger.pl"),
	 $cgi->submit(-value=>'Examine or change individual problem set data for:'), "\n";

my ($key, $id, $lname, $fname, @logins, %fullnames);
foreach $key (@availableStudents) {
	&attachCLRecord($key) || print "no Record $key\n";
	$lname = &CL_getStudentLastName($key);
	$fname = &CL_getStudentFirstName($key);
	push (@logins, $key); #array for values of popup_menu
	$fullnames{$key} = "$lname, $fname";
}

print $cgi->br,
	$cgi->popup_menu(-name=>"studentLogin", -values=>\@logins, -labels=>\%fullnames), "\n",
	$cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash), "\n",
	$cgi->hidden(-name=>'save', -value=>"OFF"), "\n",
	$cgi->hidden(-name=>'firsttime', -value=>1), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->hidden(-name=>'pCD', -value=>'1'), "\n",
	$cgi->endform(), "\n",
	"Use this to make changes for an individual student. For example, you can extend the due date for an individual student or change the status
	(correct, incorrect) or number of allowed attempts on a problem.  You will also be able to download a postscript file of
	this version of the problem set.  If you know the psvn number, e.g. from
	3 above (\"View Students Progress\"), you can enter this psvn number directly in 10 below
	(\"Examine or change database for psvn\").  Note that you will get an error message if the
	problem set has not been built for the student you select.\n";



## dataMunger form -- allows you to modify indidual versions of the problem
print heading('dataMungerPSVN', "10. Examine or modify data for an individual student in $Course:"),
	$cgi->startform(-action=>"${cgiURL}dataMunger.pl"),
	$cgi->submit(-value=>'Examine or change data for psvn:'), "\n",
	$cgi->textfield(-name=>'probSetKey', -col=>'15', -value=>'Enter PSVN'), "\n",
	$cgi->hidden(-name=>'save', -value=>'OFF'), "\n",
	$cgi->hidden(-name=>'firsttime', -value=>1), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	"Use this to make changes for an individual student. For example, you can extend the due date for an individual student or change the status
	(correct, incorrect) or number of allowed attempts of a problem.  You will also be able to download a postscript file of
	this version of the problem set.";


## profChangeDates form -- allows you to modify data for entire sets


print heading('ChangeDates', "11. Examine or modify data for all students in $Course:"),
	$cgi->startform(-action=>"${cgiURL}profChangeDates.pl"),
	$cgi->submit(-value=>'Examine or change database for entire set:'), "\n",
	$cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash),
	hiddens('user', 'key', 'course'),
	$cgi->hidden(-name=>'pCD', -value=>'1'), "\n",
	$cgi->endform(), "\n",
	"For example, you can extend the due date for the entire class or change the status
	of a problem for all students.";


$labels{alph} = 'Alphabetically';
$labels{section} = 'by section';
$labels{recitation} = 'by recitation';

print heading('PaperCopies', "12. Download paper copies for some or all students in $Course:"),
	$cgi->startform(-action=>"${cgiURL}profDownloadPaperCopies.pl"),
	$cgi->submit(-value=>'Download paper copies'), "\n",
	$cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash),
	$cgi->br, "\n",
		"Order students \n",
	$cgi->radio_group(-name=>'format', -values=>['alph','section','recitation'], -default=>'alph', -labels=>\%labels),
	hiddens('user', 'key', 'course'),
	$cgi->hidden(-name=>'pCD', -value=>'1'), "\n",
	$cgi->endform(), "\n",
	"You can download a postscript (or pfd, tex, dvi) file containing prints outs for selected students.";



# delete an entire problem set
print heading('DeleteProbSet', "13. Delete problem set(s) in $Course:"),
	$cgi->startform(-action=>"${cgiURL}profDeleteProbSet.pl"),
	$cgi->submit(-value=>'Enter Delete Problem Set Page'), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	$cgi->p, "\n",
	"You can delete Problem Sets for the whole class.";


##  housekeeping page form
print heading('Housekeep', "14. Housekeeping utilities for $Course:"),
	$cgi->startform(-action=>"${cgiURL}profHousekeeping.pl"),
	$cgi->submit(-value=>'Enter Housekeeping Page'), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	$cgi->p, "\n",
	"You can recover disk space by removing old tmp files,
	compress the gdbm database, show users currently logged in, edit the motd file and the course homepage, etc.";

}



## older scripts are archived here


## View class grades
#print heading("12. View grades for $Course:"),
#	 $cgi->startform(-action=>"${cgiURL}classGradeSummary.pl"),
#	 $cgi->submit(-value=>'View Grades'), "\n",
#	 $cgi->hidden(-name=>'course', -value=>$Course), "\n",
#	 $cgi->hidden(-name=>'user', -value=>$User), "\n",
#	 $cgi->hidden(-name=>'key', -value=>$Session_key), "\n",
#	 $cgi->endform(), "\n",
#	 $cgi->p, "\n",
#	 "You can view a summary of all grades for all students in all sets.";
#




### end of second if ( -e "${databaseDirectory}$Global::database" )
### show only build problem set part if data base does not exist


#print <<EOF;
#<HR SIZE=5 NOSHADE>
#<H3 ALIGN=CENTER>Older scripts:</H3>
#EOF
#
## oldclasslist.pl -- dumps much of the database for inspection
#print <<EOF;
#<HR>
#<FORM METHOD = "POST" ACTION= "${cgiURL}oldclasslist.pl">
#         <INPUT TYPE="SUBMIT" VALUE="View Class List">
#         <INPUT TYPE="HIDDEN" NAME="user" VALUE="$inputs{'user'}">
#         <INPUT TYPE="HIDDEN" NAME="key" VALUE="$inputs{'key'}">
#         <INPUT TYPE="HIDDEN" NAME="course" VALUE="$inputs{'course'}">
#        </FORM>
#An older version of the view class list script which allows you to view all sets and
#much of the information available in the database.
#EOF

# other scripts
#print <<EOF;
#<HR>
#Enter WeBWorK as a student
#    <p>
#    <a href="${cgiURL}profProbSet.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
#    <img src="$Global::bluesquareImgUrl"
#        border=1
#        alt=""> profProbSet.pl</a> -- not working
#
#<P><HR>
#
#
#
#
#<P>
#
#
#<P>profProbSet.pl script:
#<a href="${cgiURL}profProbSet.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
#     <B>Test a Problem Set</B></a> -- not working
#    <BR>
#
#
#
#EOF
#<H3 align="center">List of Problem Sets</H3>
# These are not really needed since there is a separate script for viewing and changing dates.
#my %setNumberHash=&getAllSetNumbersForStudentLoginHash('gage');
#&printProbSets("",\%setNumberHash);

print &htmlBOTTOM("profLogin.pl", \%inputs);

# begin Timing code
if ($logTimingData == 1) {
	my $endTime = new Benchmark;
	&Global::logTimingInfo($beginTime,$endTime,'profLogin.pl',$Course,$User);
}
# end Timing code
exit;

################################################################################
#
# HELPFUL METHODS
#
################################################################################

#returns the horizontal line, blue square, and heading for each option
#in the list, eliminating the need to repeat this code each time it is needed
#this does NOT PRINT the info, so that it can be incorporated into other text
sub heading {
        my $link = shift; #name for internal link (used for shortcuts on some pages)
        my $text = shift; #text used as the main heading

        return "\n",
                 $cgi->p, "\n",
                 $cgi->a({-name=>$link}, ''), "\n",
                 $cgi->hr({ -noshade=>undef }), "\n",
                 $cgi->h4({ -align=>'LEFT' },
                                "\n" . $cgi->img({-src=>"$Global::bluesquareImgUrl", -border=>1, -alt=>'' }) . "\n" .
                                $text ), "\n";
}

#prints hidden form fields for each of given cgi parameters
#if a given parameter does not exist, a note is placed in the html to that affect
#this does NOT PRINT the info, so that it can be incorporated into other text
sub hiddens {
	my @params = @_;
	my $out;

	foreach my $param (@params) {
		if (exists $inputs{$param}) {
			$out .= $cgi->hidden(-name=>"$param", -value=>"$inputs{$param}") . "\n";
		} else {
			$out .= $cgi->p . "\nExpected cgi parameter $param does not exist or is empty";
		}
	}
	$out;
}