#!/usr/local/bin/webwork-perl

## This file is profSortScoringFiles.pl
## It provides access to utilities for downloading scoring files
##

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################

use lib '.'; use webworkInit; # WeBWorKInitLine
use CGI qw(:standard);
use Global;
use Auth;
use strict;

# begin Timing code
use Benchmark;
my $beginTime = new Benchmark;
# end Timing code

&CGI::ReadParse;
my %inputs =%main::in;

# get information from CGI inputs  (see also below for additional information)

my $Course 			= $inputs{'course'};
my $User 			= $inputs{'user'};
my $Session_key 	= $inputs{'key'};

my @array = %inputs;
# print "inputs are @array";

# verify that information has been received
unless($Course && $User && $Session_key) {
    &wwerror("$0","The script did not receive the proper input data.","","");
}


# establish environment for this script

&Global::getCourseEnvironment($inputs{'course'});

my $cgiURL   				= getWebworkCgiURL;
my $databaseDirectory 		= getCourseDatabaseDirectory;
my $htmlURL       			= getCourseHtmlURL;
my $scriptDirectory   		= getWebworkScriptDirectory;
my $scoringDirectory        = getCourseScoringDirectory;
my $DAT                     = getDat;
my $dd                      = getDirDelim;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";
require "${scriptDirectory}$Global::SCRtools_pl";

# log access
&Global::log_info('', query_string);


my $permissionsFile = &Global::getCoursePermissionsFile($inputs{'course'});
my $permissions = &get_permissions($inputs{'user'}, $permissionsFile);
my $keyFile = &Global::getCourseKeyFile($inputs{'course'});

#verify session key
&verify_key($inputs{'user'}, $inputs{'key'}, $keyFile, $inputs{'course'});

# verify permissions are correct
if ($permissions != $Global::instructor_permissions ) {
    print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
    print &html_NO_PERMISSION;
    exit(0);
}
# get the rest of the information from the submitted form


my $totalsFile      = $inputs{'totalsFile'};
my $scrFiles        = $inputs{'scrFiles'};
my $fulFiles        = $inputs{'fulFiles'};
my $fileName        = $inputs{'fileName'};
my $action          = $inputs{'action'};
my $sortOrder       = $inputs{'sortOrder'};

## define values even if check boxes not checked
$totalsFile = 0 unless defined $totalsFile;
$scrFiles = 0 unless defined $scrFiles;
$fulFiles = 0 unless defined $fulFiles;

## if totals file is the only one selected, sort it immediately

OPTIONSWITCH: {
    if ($totalsFile == 1  and  $scrFiles == 0  and  $fulFiles == 0 and $action eq '') {
        $fileName = "${scoringDirectory}${Course}_totals.csv";
        unless (-r $fileName) {
            &wwerror("$0","The file $fileName is not readable by the webserver","","");
        }
        &sortScoringFile($fileName, $sortOrder);
       last  OPTIONSWITCH;
    }
    if ($scrFiles > 0  or  $fulFiles > 0 and $action eq '') {
        &selectFilesForm();
        last  OPTIONSWITCH;
    }
    if ($action eq 'sort') {
        &sortScoringFile("${scoringDirectory}$fileName", $sortOrder);
        last  OPTIONSWITCH;
    }
    if ($totalsFile == 0  and  $scrFiles == 0  and  $fulFiles == 0 and $action eq '') {
       &wwerror("$0","You must check at least one check box","","");
       last  OPTIONSWITCH;
    }
}

# begin Timing code
my $endTime = new Benchmark;
&Global::logTimingInfo($beginTime,$endTime,"profScoring.pl",$inputs{'course'},$inputs{'user'});
# end Timing code

exit;

## subroutines


sub selectFilesForm {

# print HTML text
print &htmlTOP("Select Scoring File for sorting");

# print navigation buttons
print qq!
<A HREF="${cgiURL}profLogin.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
!;

print <<EOF;
<h3 align="left">Sort scoring files for $Course</h3> <p>
EOF


print <<EOF;
<P>
<HR NOSHADE>
<H4 ALIGN=LEFT>
Select files:
</H4>

		<FORM METHOD = "POST" ACTION= "${cgiURL}profSortScoringFiles.pl">
         <INPUT TYPE="SUBMIT" VALUE="Sort">

EOF

 ## find the available files

opendir SCORINGDIR, $scoringDirectory or wwerror("$0","Can't open directory $scoringDirectory","","");
my @allFiles = grep !/^\./, readdir SCORINGDIR;
closedir  SCORINGDIR;

    ## sort the files

my @totalsFiles = grep /${Course}_totals\.csv$/,@allFiles;
my @scrFiles =  grep /scr\.csv$/,@allFiles;
my @fulFiles =  grep /ful\.csv$/,@allFiles;
my  @sortedNames = ();

if ($totalsFile) {
    @totalsFiles = sort @totalsFiles;
    @sortedNames = (@sortedNames,@totalsFiles);
}
if ($scrFiles) {
    @scrFiles = sort @scrFiles;
    @sortedNames = (@sortedNames, @scrFiles);
}
if ($fulFiles) {
    @fulFiles = sort @fulFiles;
    @sortedNames = (@sortedNames, @fulFiles);
}

## print list of files

print qq! <SELECT Name="fileName" >\n!;
my ($ind);
for $ind (@sortedNames) {
	print "<OPTION VALUE = \"$ind\">$ind\n";
}
    print qq!</SELECT>\n!;

 ## resume printing the rest of the form

print <<EOF;
         <INPUT TYPE="HIDDEN" NAME="user" VALUE="$inputs{'user'}">
         <INPUT TYPE="HIDDEN" NAME="key" VALUE="$inputs{'key'}">
         <INPUT TYPE="HIDDEN" NAME="course" VALUE="$inputs{'course'}">
         <INPUT TYPE="HIDDEN" NAME="action" VALUE='sort'>
         <INPUT TYPE="HIDDEN" NAME="sortOrder" VALUE="$inputs{'sortOrder'}">         
		 </FORM>
EOF
print &htmlBOTTOM("profFormEditScoringFiles.pl", \%inputs);
}



sub selectionError {
  print"content-type:\n\n<H2>Error:  You must make a selection!</H2>\n";
  print "<FORM METHOD=POST ACTION=\"${cgiURL}profSortFiles.pl\"><P>";
  print &sessionKeyInputs(\%inputs);
print <<"ENDOFHTML";
<INPUT TYPE=SUBMIT VALUE="Return to Sort Scoring Files Page">
</FORM>
ENDOFHTML
  print &htmlBOTTOM("profSortFiles.pl", \%inputs);
}

sub sortScoringFile {
    my ($fulFileName, $sortOrder) = @_;
    my $shortFileName   =   $fulFileName;
    if ($shortFileName  =~  m|$dd|)  {
        $shortFileName  =~  m|$dd([^$dd]*)$|;  ## extract filename from full path name
        $shortFileName  =   $1;
    }    
    $shortFileName      =~  s|\..*||;      ## remove extension	
    my %assocArray = &dat2aa("$shortFileName");
    &backup("$shortFileName");   ##make up to three backups	
    &aa2dat(\%assocArray,"$shortFileName", $sortOrder);     ##sort
    &sortSuccess($fulFileName);

}


sub sortSuccess {
  my ($savedFile) = @_;
  print"content-type:\n\n<H2>Success, file sorted. </H2>\n";
  print "Your file:<BR>$savedFile <BR>has been successfully sorted. The original file
  has been saved to a backup file.";
    
  print "<FORM METHOD=POST ACTION=\"${cgiURL}profScoring.pl\"><P>";
  print &sessionKeyInputs(\%inputs);
print <<"ENDOFHTML";
<INPUT TYPE=SUBMIT VALUE="Return to Scoring Page">
</FORM>
ENDOFHTML
  print &htmlBOTTOM("profSortScoringFiles.pl", \%inputs);
}
