#!/usr/local/bin/webwork-perl

## This file is profViewClasslistBD.pl
## It provides access to utilities for downloading Classlist files
##

####################################################################
# Copyright @ 1995-2000 University of Rochester
# All Rights Reserved
####################################################################

# Corrected missing html header 8-27-00 AKP

use lib '.'; use webworkInit; # WeBWorKInitLine
use CGI qw(:standard);
use Global;
use Auth;
use strict;
use File::Copy;

my $logTimingData = 0;
my $beginTime;

# begin Timing code
if ($logTimingData == 1) {
	use Benchmark;
	$beginTime = new Benchmark;
}
# end Timing code


&CGI::ReadParse;
my %inputs =%main::in;

# get information from CGI inputs  (see also below for additional information)

my $Course 			= $inputs{'course'};
my $User 			= $inputs{'user'};
my $Session_key 	= $inputs{'key'};

my @array = %inputs;
# print "inputs are @array";

# verify that information has been received
unless($Course && $User && $Session_key) {
	&wwerror("$0","The script did not receive the proper input data.","","");
}


# establish environment for this script

&Global::getCourseEnvironment($inputs{'course'});

my $cgiURL   				= getWebworkCgiURL;
my $databaseDirectory 		= getCourseDatabaseDirectory;
my $htmlURL       			= getCourseHtmlURL;
my $scriptDirectory   		= getWebworkScriptDirectory;
my $DELIM 					= $Global::delim;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::classlist_DBglue_pl";
require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";

# log access
&Global::log_info('', query_string);


my $permissionsFile = &Global::getCoursePermissionsFile($inputs{'course'});
my $permissions = &get_permissions($inputs{'user'}, $permissionsFile);
my $keyFile = &Global::getCourseKeyFile($inputs{'course'});


#verify session key
&verify_key($inputs{'user'}, $inputs{'key'}, $keyFile, $inputs{'course'});

# verify permissions are correct
if ($permissions != $Global::instructor_permissions ) {
	print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
	print &html_NO_PERMISSION;
	exit(0);
}
# get the rest of the information from the submitted form

my $format = $inputs{'format'};  #alph, section, or recitation

my $classList_ref;

if ($format eq 'section') {
		$classList_ref = getAllLoginNamesSortedBySectionThenByName();
}
elsif ($format eq 'recitation') {
		$classList_ref = getAllLoginNamesSortedByRecitationThenByName();
}
else {
		$classList_ref = getAllLoginNamesSortedByName();
}
my $contentType = 'content-type: text/html';
print "$contentType\n\n";
print output_table_in_html();
print &htmlBOTTOM('profViewClasslistBD.pl', \%inputs);

# begin Timing code
if ($logTimingData == 1) {
	my $endTime = new Benchmark;
	&Global::logTimingInfo($beginTime,$endTime,'profViewClasslistBD.pl',$Course,$User);
}
# end Timing code
exit;

sub output_table_in_html {


	my $studentLogin;
	my @outputarray =();
	my $row ='';


	foreach $studentLogin (@$classList_ref) {
		attachCLRecord($studentLogin);

		$row ='';

		my $StudentID			= CL_getStudentID($studentLogin);
		my $StudentLastName		= CL_getStudentLastName($studentLogin);
		my $StudentFirstName	= CL_getStudentFirstName($studentLogin);
		my $StudentStatus		= CL_getStudentStatus($studentLogin);
		my $StudentComment		= CL_getComment($studentLogin);
		my $ClassSection		= CL_getClassSection($studentLogin);
		my $ClassRecitation		= CL_getClassRecitation($studentLogin);
		my $StudentEmail		= CL_getStudentEmailAddress($studentLogin);

		$row =join '',$StudentID, $DELIM, $StudentLastName,	$DELIM, $StudentFirstName, $DELIM, $StudentStatus, $DELIM, $StudentComment, $DELIM, $ClassSection, $DELIM, $ClassRecitation, $DELIM, $StudentEmail, $DELIM, $studentLogin;
		push(@outputarray,$row);
	}

	my $label = "Classlist Database for $Course";
	my $outputstring = delimitedArray2html(\@outputarray, $label);
	$outputstring;
}





					