#!/usr/local/bin/webwork-perl

## fast entry for practice users to get random user

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################


use lib '.'; use webworkInit; # WeBWorKInitLine

## randPracticeLogin.pl
##
## To use, set a web link to 
##
##  href="http://my.server.name.edu/cgi-bin/webwork/system/randPracticeLogin.pl?course=MyCourse"
##
##  where MyCourse is the course ID of a course and my.server.name.edu is filed in 
##  with the name of your server.  Clicking on the link will attempt to find an open
##  practice user for that course, and log the person in.
##
## To Do:
##	* tailor problem set screen since it is also entry screen?
##	* make a log entry if we run out of practice users?
##	* do more cleaning out of code this script does not need


require 5.001;
use strict;

use Global;
use Auth;
use CGI qw(:standard);

# begin Timing code
use Benchmark;
my $beginTime = new Benchmark;
# end Timing code

&CGI::ReadParse;
my %inputs = %main::in;

my $Course	= $inputs{'course'};
my $User;
$User = 'practice' unless defined($User);
my $Passwd	= 'howdy'; # Can't fail!
#my $scriptURL = param('SCRIPTING_FILE');   # script to execute after fast relogin

# We better know what course we are going to.

&Global::getCourseEnvironment(param('course')) if param('course');

my $scriptDirectory = $Global::scriptDirectory;
my $cgiURL                  = getWebworkCgiURL();
my $databaseDirectory       = getCourseDatabaseDirectory(); #$Global::databaseDirectory;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::classlist_DBglue_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";
require "${scriptDirectory}$Global::FILE_pl";

my $Passwd_file     = &getCoursePasswordFile($Course);
my $keyFile        = &getCourseKeyFile($Course);
my $permissionsFile = &getCoursePermissionsFile($Course);
my $practiceUser = $Global::practiceUser;

my $permissions  		= &get_permissions($User,$permissionsFile);

# If a valid key is specified, we've just changed password
# so no need to re-authorize

## No course specified?  You lose.
unless($Course) {
    my $error_msg = remote_host().'   '.user_agent().'   '.query_string();
    &wwerror("$0, missing course data. Refered by:" . referer(),
        "The script did not receive the proper input data. No WeBWorK course was defined",'','',$error_msg
	);
}

# Find practice users in system
my @currentUsers = get_current_users_from_passwordDB($Passwd_file);
my @currentPracticeUsers = grep /^$practiceUser/, @currentUsers;

# Remove practice users who are already logged in

my @currentAvailablePracticeUsers;

if (-e $keyFile) {							## if the keys files does not exist, they can be no conflict
	my @activeUsers = get_active_users_from_keysDB($keyFile);
	my @activePracticeUsers = grep /^$practiceUser/, @activeUsers;
	my (%mark,$user);
	grep($mark{$_}++,@activePracticeUsers);
	@currentAvailablePracticeUsers = grep(!$mark{$_},@currentPracticeUsers);
} else {
	@currentAvailablePracticeUsers = @currentPracticeUsers;
}
	
my $numberAvailable = @currentAvailablePracticeUsers;

if($numberAvailable == 0) {
	&practice_failure_form;
	exit;
}

# Randomly pick one
$User = $currentAvailablePracticeUsers[int(rand $numberAvailable)];

my $Session_key = &new_key($User, $keyFile);
&record_login;

my $studentLogin=$User;
$inputs{'course'} = $Course;
$inputs{'user'} = $User;
$inputs{'key'} = $Session_key;

## check that database exists
unless ( -e "${databaseDirectory}$Global::database" ) {
    wwerror($0, "No problem sets have been built yet");
}

# obtain list of setNumbers

my %setNumberHash=&getAllSetNumbersForStudentLoginHash($studentLogin);

# get one PSVN and determine the students name.
my @PSVNs = values %setNumberHash;

# this is so the correct emailaddress gets sent to feedback
$inputs{'probSetKey'}=$PSVNs[0];

attachCLRecord($studentLogin);

my $studentName=&CL_getStudentName($studentLogin);
my $currentDate = &formatDateAndTime(time);

# get information on open/due/answer dates for each problem and prepare
# HTML output;

my @SetNumberKeys =  keys(%setNumberHash);
my @problemDates = ();
my $problemDateLine;
my ($probSetKey,$odts,$ddts,$adts,$timeNow,$DueDate,$AnswerDate,$OpenDate);
my $sortedSetNumber;
my $SetNumber;
my %problemDateLines =();
my %dueTimes =();

foreach $SetNumber(@SetNumberKeys) {
    $probSetKey=$setNumberHash{$SetNumber};
    &attachProbSetRecord($probSetKey);
    $odts=&getOpenDate($probSetKey);
    $ddts=&getDueDate($probSetKey);
    $dueTimes{$SetNumber} = $ddts;
    $adts=&getAnswerDate($probSetKey);
    $timeNow = time;

    $DueDate=&formatDateAndTime($ddts);
		$AnswerDate = &formatDateAndTime($adts);
		$OpenDate =  &formatDateAndTime($odts);

# prepare message based on current time relative to the Open, Due and Answer dates.
    $problemDateLine = "";
    $problemDateLine = "<OPTION VALUE= \"$probSetKey\">";

my $sssname = $SetNumber;
$sssname =~ s/_/ /g;
    $problemDateLine .= "$sssname";

     ($timeNow < $odts ) &&                             # beforeOpenDateMsg
        do {$problemDateLine .= " --- Before open date -- Open date is: $OpenDate";};
        ( $odts <= $timeNow ) && ($timeNow < $ddts) &&  # afterOpenDateMsg
                do {$problemDateLine .=  " --- OPEN--  Due date is: $DueDate";};
        ( $ddts <= $timeNow ) && ($timeNow < $adts) &&  #  afterDueDateMsg
                do {$problemDateLine .=  " --- CLOSED -- Answers available on: $AnswerDate";};
        ( $adts <= $timeNow ) &&                        # afterAnsDateMsg
                do {$problemDateLine .=  " --- CLOSED --  answers available.";};


    $problemDateLines{$SetNumber} =$problemDateLine;

}

## Sort setnumbers by due date

sub by_due_date
	{
		$timeNow = time;
		if ( ($dueTimes{$a} <= $timeNow) and ($dueTimes{$b} <= $timeNow) )
			{
				($dueTimes{$a} <=> $dueTimes{$b})
					or
            ($a cmp $b)
					}
		elsif ( ($dueTimes{$a} > $timeNow) and ($dueTimes{$b} > $timeNow) )
			{
				($dueTimes{$a} <=> $dueTimes{$b})
					or
            ($a cmp $b)
					}
		else
			{
				$dueTimes{$b} <=> $dueTimes{$a}
			}
		
	}

my @sortedSetNumberKeys = sort by_due_date keys(%problemDateLines);

foreach $sortedSetNumber(@sortedSetNumberKeys)
	{
		push (@problemDates, $problemDateLines{$sortedSetNumber});
	}


# begin printing the WELCOME page
print &htmlTOP("WeBWorK Welcome Page");

# print navigation buttons
print qq!
<A HREF="${cgiURL}login.pl?user=$User&key=$Session_key&course=$inputs{'course'}">
<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
!;

my $course_disp = $inputs{'course'};
$course_disp =~ s/_/ /g;
print <<"ENDOFHTML1";
<H3 ALIGN="LEFT">WeBWorK welcomes $studentName for class $course_disp.</H3>

From this page you can view and answer a problem set, download a printed version
of the entire problem set in postscript or pdf format, or view a summary
of your homework grades.  If a set is closed, you can still do the problems and
WeBWorK will tell you whether your answers are correct or incorrect, but your
answers will not be recorded.
ENDOFHTML1

my $prof_switch = '';

# FORM for accessing the problem sets via computer
print "<form METHOD = \"POST\" action=\"$Global::welcomeAction_CGI\">";
print "<HR NOSHADE>";
print "The date is currently: <B>$currentDate </B>.";
#print "<P>Select a problem set to work on, or to print:<BR> <SELECT NAME = \"probSetKey\" SIZE = \"7\" $prof_switch>\n";
print "<P>Select a problem set to work on, or to print:<BR> <SELECT NAME = 'local_psvns' SIZE = \"7\" $prof_switch>\n";

print join("\n\n", @problemDates),"\n";  # include open/due/answer dates
                     # prepared above
print "</SELECT>\n";
print &sessionKeyInputs(\%inputs);
print "<BR>";
print "<BR><input type=\"submit\"name = \"action\" value=\"Do problem set\">";
print "<BR><input type=\"submit\"name = \"action\" value=\"Get hard copy\">";
#print " in: <INPUT TYPE=RADIO NAME=\"downloadType\" VALUE=\"ps\" CHECKED><B>PostScript</B> form&nbsp;&nbsp;";
#print "<INPUT TYPE=RADIO NAME=\"downloadType\" VALUE=\"pdf\" ><B>PDF</B> form\n";
print " in: <INPUT TYPE=RADIO NAME='downloadType' VALUE='pdf' CHECKED><B>PDF</B> form&nbsp;&nbsp;";
print "<INPUT TYPE=RADIO NAME='downloadType' VALUE='ps'><B>PostScript</B> form\n";
print "<BR><INPUT TYPE=CHECKBOX NAME=\"ShowAns\" VALUE=\"1\"> Show answers in hard copy IF the answers are available.\n" ;
print "</FORM>\n ";

# FORM for viewing student summary

print "<FORM METHOD = \"POST\" action=\"${Global::cgiWebworkURL}studentSummary.pl\">\n ";
print "<HR NOSHADE>Obtain a summary of your WeBWorK scores:<BR>\n ";

print &sessionKeyInputs(\%inputs);
print "<input type=\"submit\" value=\"Get Summary\">";
print "</FORM>";



print &htmlBOTTOM("practiceUser.pl",\%inputs);

############################################################

# begin Timing code
my $endTime = new Benchmark;
&Global::logTimingInfo($beginTime,$endTime,"login.pl",$Course,$User,remote_host(),user_agent());
# end Timing code

exit;

####################

sub failure_form {
    print &htmlTOP('Login Invalid', $Global::background_warn_url),
	h1('Login Invalid'),
	"Sorry ", b($User), ", is not valid.  This should not happen; please contact your instructor.",
	start_form('POST', url),
	hidden('course'),
	submit('Try again'),
	end_form, end_html;
}

sub practice_failure_form {

    ## now print form
    print &htmlTOP('Login Failed', $Global::background_warn_url),
	  h1('Login Failed'),
	  'Sorry, the ', b('practice'),' accounts are currently being used by other users. ',
	  'You can try to login at a later time.<p> ',
	'Please inform your instructor that we temporarily used up all of the slots for practice users',
	  start_form('POST', url),
	  hidden('course'),
	  submit('Try again'),
	  end_form, end_html;
}

sub record_login {

	&Global::getCourseEnvironment($Course);
	my $logsDirectory = getCourseLogsDirectory();
	my $logfile = "${logsDirectory}login.log";
	my $timestamp = scalar localtime;
	
	$timestamp =~ /^\w+\s(.*)\s/;
	$timestamp = $1;
	
	open(LOG, ">>$logfile") || warn "$0: can't write to $logfile!\n";
	print LOG "$timestamp $User on ", remote_host(),
		" (", user_agent(), ")\n";
	close(LOG);
	chmod 0660, "$logfile";
}
