#!/usr/local/bin/webwork-perl

## $Id: scoreProbSet.pl,v 1.6 2002/10/16 13:06:36 apizer Exp $

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################


# This file is scoreProbSet.pl

use lib '.'; use webworkInit; # WeBWorKInitLine

use strict;
use Global;
use Auth;
use CGI qw(:standard);

# Timing code
use Benchmark;
my $beginTime = new Benchmark;
# end Timing code

my $cgiMode = 0; #  was script called from command line or via CGI?
my $batchMode = 0;
my $page ='';
$cgiMode = 1 if $ENV{'SERVER_SOFTWARE'};

my %inputs;
my ($classID, $setNumber,$User,$Session_key,$sortOrder);
my $appendTotalsQ = 0;

if ( $cgiMode == 1 ) {
    &CGI::ReadParse;
    %inputs = %main::in;

    # get information from CGI inputs  (see also below for additional information)
    $classID 	 = $inputs{'course'};
    $setNumber = $inputs{'setNo'};
    $appendTotalsQ = $inputs{'appendTotals'} if defined $inputs{'appendTotals'};
    $batchMode = $inputs{'batchMode'};
    $sortOrder = $inputs{'sortOrder'};
    $page = $inputs{'page'};
    $User	 = $inputs{'user'};
    $Session_key = $inputs{'key'};

    unless ($classID && $User && $Session_key) {
        &wwerror("$0","The script scoreProbSet.pl did not receive the proper input data.","","", query_string());
    }
}
else {
    # get information from command line arguments
    ($classID, $setNumber) = @ARGV;
    $appendTotalsQ = 1;
	if (@ARGV != 2)  {
		print "\nSyntax is scoreProbSet.pl courseID setName \n";
    	print "      (e.g. scoreProbSet.pl MTH140A 7 ) \n\n";
    	exit(0);
	}
}


# CAUTION:  FOR THIS SCRIPT TO WORK CORRECTLTY,  ALL STUDENTS
#           MUST HAVE BEEN ASSIGNED THE SAME NUMBER OF PROBLREMS

# print "set is $setNumber";
# print "classID is $classID";

&getCourseEnvironment($classID);

my $scriptDirectory   = getWebworkScriptDirectory;
my $databaseDirectory = getCourseDatabaseDirectory;
my $templateDirectory = getCourseTemplateDirectory;
my $scoringDirectory  = getCourseScoringDirectory;
my $cgiURL   		  = getWebworkCgiURL;
my $DAT = &getDat;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::classlist_DBglue_pl";
require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}$Global::SCRtools_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";

#   verify the identity of the user if from cgi
if ( $cgiMode == 1 ) {
	my $keyFile = &Global::getCourseKeyFile($classID);
	# log access
	&Global::log_info('', query_string);

	&verify_key($User, $Session_key, $keyFile, $classID);
	my $permissionsFile = &Global::getCoursePermissionsFile($classID);
	my $permissions = &get_permissions($User, $permissionsFile);

	if ($permissions != $Global::instructor_permissions ) {
		print "permissions = $permissions instructor_permissions= $Global::instructor_permissions\n";
		print &html_NO_PERMISSION;
		exit(0);
	}
}

# get list of problems and values from the database.  Warn user if different students
# have different numbers of problems or different valuse for the same problem

my ($warningMsg,$dueDate, $problemListref,$problemValueListref)
	= &readProblemsAndValuesFromDB($setNumber,0,$cgiMode,$batchMode);

if (( $cgiMode == 1 ) and ($warningMsg ne '')) { &warningMsgPage(\%inputs,$warningMsg);}


my @problemList = @$problemListref;
my @problemValueList = @$problemValueListref;
my $noOfProbs = @problemList;


$dueDate = &formatDateAndTime($dueDate);	##originally dueDate is in the form 4/15/96 at 2:00 AM

my $dueTime=$dueDate;
$dueDate=~s|^\s*||;
$dueDate=~s|\s*at.*$||;
$dueTime=~s|^.*at\s*||;
$dueTime=~s|\s*$||;

#print "$dueDate\n";
#print "$dueTime\n";

&get_raw_scores('db', $setNumber, $dueDate, $dueTime);

my $scrFileName = "s${setNumber}scr";

if ( $cgiMode == 1  and $sortOrder ne 'AbySec')  {
    my %assocArray = &dat2aa("$scrFileName");
    &aa2dat(\%assocArray,"$scrFileName", $sortOrder);                           ##sort
}    

my $fullTotalsFileName = "${scoringDirectory}${classID}_totals.${DAT}";
my $fullScrFileName = "${scoringDirectory}s${setNumber}scr.${DAT}";

if ( $cgiMode == 0 ) {
	print "\n Number of Probs is $noOfProbs\n";
	print "\n\nDo you want to total the scores in the score file for this set,\n";
	print "$fullScrFileName, \n";
	print "and append these totals to the cummulative totals file,\n";
	print "$fullTotalsFileName, \n\n";
	print "We recommend that you do\? (Y or N)\n";

	my $ans;

	$ans=<STDIN>;
	chomp($ans);
	unless (($ans eq "Y") || ($ans eq "y")) {$appendTotalsQ = 0;}
}

if ($appendTotalsQ == 1) {

    my $totalsFileName = "${classID}_totals";
    &total_score($scrFileName,$setNumber);

    my $gid = $Global::numericalGroupID;
    $gid = $Global::numericalGroupID;    ## hack to remove warning message about $Global::numericalGroupID used only once

    unless (-e "$fullTotalsFileName") {
	    &createFile("$fullTotalsFileName", $Global::scoring_files_permission, $gid);
    }
    unless (-w "$fullTotalsFileName") {
    	&wwerror("$0","Can't write to the file $fullTotalsFileName.");
    }
    &append_score_db($totalsFileName,$scrFileName);
    &delete_columns ($totalsFileName, -$noOfProbs-1, -2, 0);
    if ( $cgiMode == 1  and $sortOrder ne 'AbySec')  {
        my %assocArray = &dat2aa("$totalsFileName");
        &aa2dat(\%assocArray,"$totalsFileName", $sortOrder);                           ##sort
    } 
}


if ( $cgiMode == 1 ) {&finishedMsgPage(\%inputs);}



sub finishedMsgPage {

my ($inputref)  = @_;
my %inputs = %$inputref;
# print HTML text
print &htmlTOP("Scoring Utilities");

# print navigation buttons ${cgiURL}profScoring.pl
print qq!
<A HREF="${cgiURL}profScoring.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
!;

print <<EOF;
<HR><BR>
<h3 align="left">WeBWorK scoring message for $classID:</h3>
EOF

print "Set $setNumber has been scored and the updated scoring files have been saved in the scoring directory
$scoringDirectory .<BR> <BR>
To return to the scoring page select the \"Up\" buttom.
<BR> To return to the Professor page select \"Professor page \". ";

print <<EOF;
<P>

		<FORM METHOD = "POST" ACTION= "${cgiURL}profLogin.pl">
         <INPUT TYPE="SUBMIT" VALUE="Professor page">

EOF
    
# resume printing the rest of the form

print <<EOF;
         <INPUT TYPE="HIDDEN" NAME="user" VALUE="$inputs{'user'}">
         <INPUT TYPE="HIDDEN" NAME="key" VALUE="$inputs{'key'}">
         <INPUT TYPE="HIDDEN" NAME="course" VALUE="$inputs{'course'}">    
         </FORM>     
EOF


print &htmlBOTTOM("scoreProbSet.pl", \%inputs);

} #end of finished Page


sub warningMsgPage {

my ($inputref,$warningMsg)  = @_;
my %inputs = %$inputref;
# print HTML text
print &htmlTOP("Scoring Utilities");

# print navigation buttons
print qq!
<A HREF="${Global::cgiWebworkURL}profScoring.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
!;

print <<EOF;
<HR><BR>
<h3 align="left">WeBWorK scoring WARNING message for $classID:</h3>
EOF
print "<pre>";
print $warningMsg;
print "</pre>";
print "To Continue scoring set $setNumber, check the \"Continue\" check box and then select
\"Continue scoring set $setNumber\". You can ignore the warning messages that will result.<BR> To Quit and return to the Scoring page, select the \"Up\"
buttom.";

## profScoring form -- allows you to score problem sets
print <<EOF;
<P>

		<FORM METHOD = "POST" ACTION= "${cgiURL}scoreProbSet.pl">
         <INPUT TYPE="SUBMIT" VALUE="Continue scoring set $setNumber">

EOF


    print qq{
		<BR><INPUT TYPE=CHECKBOX NAME="batchMode" VALUE=1 >Continue 
	    };
    
# resume printing the rest of the form

print <<EOF;
         <INPUT TYPE="HIDDEN" NAME="user" VALUE="$inputs{'user'}">
         <INPUT TYPE="HIDDEN" NAME="key" VALUE="$inputs{'key'}">
         <INPUT TYPE="HIDDEN" NAME="course" VALUE="$inputs{'course'}"> 
         <INPUT TYPE="HIDDEN" NAME="setNo" VALUE="$inputs{'setNo'}">
         <INPUT TYPE="HIDDEN" NAME="appendTotals" VALUE="$appendTotalsQ">              
         <INPUT TYPE="HIDDEN" NAME="sortOrder" VALUE="$inputs{'sortOrder'}">
         </FORM>     
EOF


print &htmlBOTTOM("scoreProbSet.pl", \%inputs);
exit;
} #end of warning Page






