################################################################################
# WeBWorK
# 
# Copyright (c) 1995-2001 WeBWorK Team, University of Rochester
# All rights reserved
# 
# $Id: db3_tie.pl,v 1.5 2001/11/17 17:40:22 apizer Exp $
################################################################################

use BerkeleyDB;
use strict;

my $TieMaxTries = 30;     # try to tie() 30 times, then fail
my $TieSleepInterval = 2; # sleep 2 sec after failed tie()
 
sub tie_hash {
	my($fh, $db_obj_ref, $hash_ref, $file_name, $mode_flag, $file_mode) = @_;
	# \$fh and \$db_obj_ref are not used by this routine.  Other databases may
	# require them. They are present to maintain a consistent interface
	
	my $db3_env = new BerkeleyDB::Env
		-Flags => DB_INIT_LOCK|DB_INIT_MPOOL,
		-LockDetect => DB_LOCK_DEFAULT;
	
	my $open_flag = lc $mode_flag eq 'r' ? DB_RDONLY : DB_CREATE;
	
	# try to achieve tie $Max_Tries times, with a wait of $Sleep_interval
	# between attempts.  Returns the database object to use for locking.
	foreach my $i (1..$TieMaxTries) {
		return if tie %$hash_ref, "BerkeleyDB::Hash",
			-Filename => $file_name,
			-Flags => $open_flag,
			-Mode => $file_mode,
			-Env => $db3_env;
		sleep $TieSleepInterval;
	}
	
	# We've exhausted all tries. Time to crap out.
	&wwerror($0, "Can't tie $file_name: $! ... $BerkeleyDB::Error");
}

sub untie_hash {
	my($fh, $db_obj_ref, $hash_ref, $file_name) = @_;
	## \$fh and \$db_obj_ref are not used by this routine.  Other databases may
	## require them. They are present to maintain a consistent interface
	
	(tied %$hash_ref)->db_sync();
	
	if(untie %$hash_ref) {
		# success!
		return 1;
	} else {
		wwerror($0, "Can't untie database $file_name");
	}
}

1;
